package unisa.gps.etour.control.GestioneBeniCulturali;

import java.util.Date;
import unisa.gps.etour.bean.BeanBeneCulturale;

/**
 * Questa classe ha il compito di controllare i dati di un bene culturale.
 * Vengono effettuati vari controlli di consistenza quali la lunghezza di stringhe,
 * riferimenti nulli, tipi dinamici errati.
 *
 * @author De Simone Michelangelo
 * @version 0.1
 * 
 *  &#169; 2007 eTour Project - Copyright by SE@SA Lab DMI - University of Salerno
 */
public class ControlloBeniCulturali
{
	/**
	 * 
	 * Effettua il controllo di consistenza richiamando gli opportuni metodi.
	 * Questo metodo "wrappa" tutti gli altri metodi di controllo con una sola chiamata.
	 * Al primo valore false il flusso è interrotto.
	 *
	 * @param pBC Il bean del bene culturale da controllare
	 * @return boolean Il risultato del controllo: true se OK, false altrimenti
	 */
	public static boolean controllaDatiBeneCulturale(BeanBeneCulturale pBC)
	{
		// Questo metodo controlla il parametro d'ingresso nei casi di
		// riferimento nullo o tipo dinamico errato
		if (pBC == null || !(pBC instanceof BeanBeneCulturale))
			return false;
		
		// Questo metodo controlla se l'Id del BeanBeneCulturale passato come parametro
		// è valido o meno
		if (!controllaIdBeneCulturale(pBC.getId()))
			return false;
		
		// Questo metodo controlla gli oggetti Date contenuti nel BeanBeneCulturale
		// In quanto tale viene effettuato un controllo di riferimento nullo ed un controllo
		// sul tipo dinamico.
		if (!controllaDateBeneCulturale(pBC))
			return false;
		
		// Questo metodo controlla tutti i campi presenti nel BeanBeneCulturale, alla ricerca
		// di eventuali riferimenti nulli
		if (!controllaDatiNulli(pBC))
			return false;
		
		// Controlla la corretta lunghezza delle stringhe, nel caso specifico il CAP dev'essere
		// per forza di cose di cinque cifre, mentre la provincia di DUE.
		// TODO: Da completare
		if (pBC.getCap().length() != 5 || pBC.getProvincia().length() != 2 )
			return false;
		
		return true;
	}

	/**
	 * 
	 * Controlla la presenza di dati nulli in un bean bene culturale.
	 * Il controllo viene effettuato su tutti i campi del bean.
	 *
	 * @param pBC Il bean bene culturale da controllare
	 * @return boolean Il risultato del controllo: true se non ci sono riferimenti nulli; false altrimenti
	 */
	public static boolean controllaDatiNulli(BeanBeneCulturale pBC)
	{
		if (pBC.getCap() == null || pBC.getCitta() == null || pBC.getDescrizione() == null || pBC.getGiornoChiusura() == null ||
				pBC.getLocalita() == null || pBC.getNome() == null || pBC.getOrarioApertura() == null ||
				pBC.getOrarioChiusura() == null || pBC.getProvincia() == null || pBC.getTelefono() == null || pBC.getVia() == null)
			return false;

		return true;
	}
	
	/**
	 * 
	 * Controlla la consistenza delle date presente all'interno dei BeanBeneCulturale.
	 * Il controllo viene effettuato esclusivamente sugli oggetti, mentre non viene effettuato
	 * alcun controllo di validita' su  una data in quanto un bene culturale potrebbe avere anche
	 * date successive a quella odierna (vedi ad esempio data apertura di mostre).
	 *
	 * @param pBC Il BeanBeneCulturale del quale controllare le date
	 * @return boolean Il risultato del controllo: true se le date hanno consistenza; false altrimenti
	 */	
	public static boolean controllaDateBeneCulturale(BeanBeneCulturale pBC)
	{
		if (pBC.getOrarioApertura() == null || !(pBC.getOrarioApertura() instanceof Date) ||
				pBC.getOrarioChiusura() == null || !(pBC.getOrarioChiusura() instanceof Date))
			return false;
		
		return true;
	}
	
	/**
	 * 
	 * Controlla l'ID del BeanBeneCulturale
	 *
	 * @param pId L'ID del BeanBeneCulturale da controllare
	 * @return boolean Il risultato del controllo: true se l'ID è corretto; false altrimenti
	 */	
	public static boolean controllaIdBeneCulturale(int pId)
	{
		return (pId > 0);
	}
}
