﻿package unisa.gps.etour.control.fuzzy;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Scanner;

public class Fuzzy
{
	/**
	 * Classe che implementa i metodi Fuzzy utilizzati per calcolare la
	 * Categoria di appartenenza di un Punto Di Ristoro o di un Bene Culturale.
	 */

	/**
	 * Metodo per il calcolo della rilevanza di un termine.
	 * 
	 * @param distanza: Contiene la distanza del termine dalla categoria
	 *            analizzata
	 * @param pMaxDist: Contiene la distaza massima di tutti i termini in tutte
	 *            le categorie.
	 * @return Restituisce il rilevanza del termine nella categoria.
	 */
	private static float rilevanza(float distanza, float pMaxDist)
	{
		return (distanza / pMaxDist);

	}

	/**
	 * Metodo per il calcolo della distanza fra un termine ed una categoria
	 * 
	 * @param pTermine: Contiene il termine analizzato
	 * @param pTotTermini: Tabelle delle frequenze totali dei termini.
	 * @param frequenzaTesto: Tabella dei termini del testo analizzato.
	 * @param pCategoria: Categoria analizzata.
	 * @return Restituisce la distanza del termine pTermine dalla categoria
	 *         pCategoria
	 */
	private static float distanza(String pTermine,
			Hashtable<String, float[]> pTotTermini, float frequenzaTesto,
			Categoria pCategoria)
	{

		// la prima variabile tiene traccia della frequenza di un termine
		// relativa ad una categoria
		// la seconda tiene traccia della fraquenza di un termine per tutte le
		// categorie
		float[] frequenzaCategoria = new float[3], frequenzaTotale = new float[3];

		if (pCategoria.esisteTermine(pTermine))// se il termine è presente
		// nella categoria
		{
			// la sua frequenza nella categoria è uguale alla frequenza presente
			// nella base di conoscenza
			// più la frequenza nel testo analizzato
			frequenzaCategoria = pCategoria.getVal(pTermine);
			frequenzaCategoria[0] += frequenzaTesto;
		}
		else
		{

			// altrimenti è uguale alla frequenza del termine nel testo
			// analizzato
			frequenzaCategoria[0] = frequenzaTesto;
		}
		if (pTotTermini.get(pTermine) != null)// se il termine esiste nella
			// tabella dei termini totali
			frequenzaTotale = pTotTermini.get(pTermine);// prelava il suo valore

		frequenzaTotale[0] += frequenzaTesto;// la frequenza totale è data
		// dalla frequenza nel testo
		// analizzatp
		// più l'eventuale frequenza salvata in memoria nella tabella dei
		// termini totali

		return (frequenzaCategoria[0] / frequenzaTotale[0]);// la distanza è
		// uguale alla
		// frequenza nella
		// categoria
		// fratto al frequenza totale

	}

	/**
	 * Metodo per il calcolo della distanza di un termine da una categoria.
	 * Utilizzato per il training
	 * 
	 * @param pTermine termine da analizzare
	 * @param pCategoria categoria da cui ci si deve calcolare la distanza
	 * @param pTotTermini tabella dei termini totali
	 * @return restituisce la distanza di un termine da una categoria
	 */
	private static float distanza(String pTermine, Categoria pCategoria,
			Hashtable<String, float[]> pTotTermini)
	{
		return ((pCategoria.getVal(pTermine))[0] / (pTotTermini.get(pTermine))[0]);
	}

	/**
	 * Implementazione di una funzione T-Norm
	 * 
	 * @param a primo valore
	 * @param b secondo valore
	 * @return restituisce il valore calcolato tramite una funzione T-Norm
	 */
	private static float tNorm(float a, float b)
	{
		return ((a * b) / ((2 - ((a + b) - (a * b)))));
		// return Math.min(a, b);
		// return a*b;
		// return Math.max(0,a+b-1);
	}

	/**
	 * Implementazione di una funzione S-Norm
	 * 
	 * @param a primo valore
	 * @param b secondo valore
	 * @return restituisce il valore calcolato tramite una funzione S-Norm
	 */
	private static float sNorm(float a, float b)
	{
		return ((a + b) / (1 + a * b));
		// return Math.max(a, b);
		// return (a+b-(a*b));
		// return Math.min(1,1+b);
	}

	/**
	 * Calcolo della similarity fra una categoria ed un testo dato in input
	 * 
	 * @param pTermini tabella dei termini del testo. Esso deve contenere per
	 *            pgni termine i valori di rilevanza e appartenenza situati nel
	 *            vettore di float nelle posizioni 1 e 2.
	 * @return restituisce un valore numerico che indica la similarity di un
	 *         testo con la categoria su cui ci si è calcolati i valori di
	 *         rilevanza e appartenenza
	 */
	private static float similarity(Hashtable<String, float[]> pTermini)
	{
		float sum = 0;// valore da restituire

		for (float[] val : pTermini.values())// per tutti gli elementi della
		// tabella
		{
			sum += (tNorm(val[1], val[0])) / (sNorm(val[1], val[0]));// effettua
			// una somma fra i valori dati dalla divisione della funzione T-Norm
			// con la funzione S-Norm effettuate
			// fra rilevanza e appartenenza
		}

		return sum;
	}

	/**
	 * Metodo per il calcolo dell'appartenenza di un testo ad una categoria
	 * 
	 * @param val indica la similarity di un testo con una categoria
	 * @param maxSimilarity indica la massima similarity ritrovata
	 * @return restiuisce un valore nell'intervallo [0,1] che indica il grado di
	 *         appartenenza del testo alla categoria x
	 */
	private static float appartenenza(float val, float maxSimilarity)
	{
		return (val / maxSimilarity);

	}

	/**
	 * Metodo per eliminare da un teso i caratteri speciali e per riportare
	 * tutte le lettere maiuscole in minuscolo
	 * 
	 * @param pStr stringa da trasformare
	 * @return restiuisce il testo in minuscolo e senza caretteri speciali
	 */
	private static String replaceAndLower(String pStr)
	{
		pStr = pStr.toLowerCase();
		pStr = pStr.replace(",", " ");
		pStr = pStr.replace(".", " ");
		pStr = pStr.replace("!", " ");
		pStr = pStr.replace("?", " ");
		pStr = pStr.replace("'", " ");

		return pStr;
	}

	/**
	 * Metodo che consente di ritrovare la categoria appartenete ad un testo
	 * 
	 * @param pDescrizione testo da analizzare
	 * @return restuisce una stringa indicante la categoria
	 * @throws RemoteException
	 */
	public static String calcolaCategoria(String pDescrizione)
			throws RemoteException
	{
		if ((pDescrizione == null) || (pDescrizione.equals("")))
			return "NULL";
		String testo = pDescrizione;
		// tabella dei termini associata al testo da analizzare. Conterrà i
		// valori di frequenza, rilevamza, appartenenza per ogni termine
		Hashtable<String, float[]> datiTesto = new Hashtable<String, float[]>();
		// tabella delle categorie, conterrà per ogni categoria il valore di
		// similarity e di appartenenza del testo
		Hashtable<String, float[]> testoCategoria = new Hashtable<String, float[]>();
		testo = replaceAndLower(testo);// elimina caratteri spaeciali e
		// restituisce il testo rimpiazzando le
		// maiuscole con le minuscole
		String[] testoSplit = testo.split(" ");
		for (int i = 0; i < testoSplit.length; i++)
		{// per ogni termine del testo
			float[] toPut = new float[3];// valori da assegnare alla stringa
			// nella tabella hash
			float[] valTmp;// variabile temporanea contenente i valori
			// associati alla stringa se già presente nella tabelle hash
			// se la stringa è presente nella tabella preleva i valori
			// e li incremente di uno salvandoli in to put
			// altrimenti inizializza la nuova stringa con frequenza = 1
			if (testoSplit[i].length() <= 3)// si eliminano termini poco
				// rilevanti come da, del, fra...
				continue;
			if (esiste(testoSplit[i], datiTesto))// se il termine analizzato
			// è presente nella tebella dei termini del testo analizzato
			{
				// si preleva il valore di frequenza presente nella tabella e vi
				// si aggiunge uno
				valTmp = datiTesto.get(testoSplit[i]);
				toPut[0] = valTmp[0];
				toPut[0] += (float) 1 / testoSplit.length;

			}
			else
			{
				toPut[0] = (float) 1 / testoSplit.length;
				// altrimenti si inizializza il valore di frequenza ad uno
				// fratto il numero di termini totali ( frequenza relativa )
			}
			// si inserisce la nuova entrata nella tabella
			// System.out.println(toPut[0]);
			datiTesto.put(testoSplit[i], toPut);
		}

		// si cerca di aprire la base di conoscenza
		ElencoCategorie elenco;
		try
		{
			elenco = apriElenco();
		}
		catch (ClassNotFoundException e)// errore nell'apertura del file kb.sbt
		{
			throw new RemoteException(
					"La base di conoscenza è assente o danneggiata!");
		}
		catch (IOException e)
		{
			throw new RemoteException(
					"La base di conoscenza è assente o danneggiata!");
		}

		// si preleva dalla basei di conoscenza la tabella dei termini totali
		Hashtable<String, float[]> totTermini = elenco.getTotTermini();
		float maxSimilarity = -1;// mantiene il valore di massima similarity
		for (String nomeCategoria : elenco.Categorie())
		{// per tutte le categorie presenti nella base di conoscenza
			float[] toPut = new float[3];// valori da assegnare alla stringa
			// nella tabella hash
			for (Enumeration<String> val = datiTesto.keys(); val
					.hasMoreElements();)
			{// per tutti gli elementi della tabella dei termini del testo
				String termine = val.nextElement();
				// si preleva il valore di un termine
				float[] tmp = datiTesto.get(termine);
				// si calcola distanza e rilevanza
				tmp[1] = distanza(termine, totTermini, tmp[0], elenco
						.getCategoria(nomeCategoria));
				tmp[2] = rilevanza(tmp[1], elenco.getMaxDist());
				datiTesto.put(termine, tmp);
			}
			// ci si calcola la similarity una volta terminata l'analisi di
			// tutti i termini con una categoria
			toPut[0] = similarity(datiTesto);
			testoCategoria.put(nomeCategoria, toPut);
			if (maxSimilarity < toPut[0])// si aggiorna il valore di massima
			// similarity se necessario
			{
				maxSimilarity = toPut[0];
			}
		}

		for (String nomeCategoria : elenco.Categorie())
		{// per ogni categoria
			// si preleva il valore di similarity del testo con la categoria
			// analizzata
			float[] tmp = testoCategoria.get(nomeCategoria);
			tmp[1] = appartenenza(tmp[0], maxSimilarity);// ci si calcola
			// l'appartenenza
			// del testo alla similarity
			testoCategoria.put(nomeCategoria, tmp);// si salva tutto nella
			// tabella delle categorie
		}

		return maxAppartenenza(testoCategoria);// si ritorna in output il nome
		// della categoria
		// con il massimo grado di appartenenza
	}

	/**
	 * Metodo per ricercare la categoria con cui il testo ha il massimo grado di
	 * appartenenza
	 * 
	 * @param pTestoCategoria tabella delle categorie associate al testo
	 * @return restiuisce una stringa indicante il nome della categoria con cui
	 *         il testo ha il massimo grado di appartenenza
	 */
	private static String maxAppartenenza(
			Hashtable<String, float[]> pTestoCategoria)
	{
		String toReturn = null;// valore di ritorno
		float max = -1;// valore di appartenenza massimo

		for (Enumeration<String> elm = pTestoCategoria.keys(); elm
				.hasMoreElements();)
		{// per tutte le categorie della tabella delle categorie del testo
			String categoria = elm.nextElement();
			// si prelevano i valori di similarity e appartenenza associati alla
			// categoria
			float[] tmp = pTestoCategoria.get(categoria);
			if (tmp[1] > max)
			{// se il grado di appartenenza appartenenza appena caricato
				// è maggiore del precedente si aggiornano i dati max e toReturn
				toReturn = categoria;
				max = tmp[1];
			}

		}
		return toReturn;
	}

	/**
	 * Metodo utilizzato per verificare se un termine è presente nella tabella
	 * dei termini del testo
	 * 
	 * @param pStr termine da analizzare
	 * @param pTable tabelle dei termini del testo
	 * @return restituisce vero se il termine esiste falso altrimenti
	 */
	private static boolean esiste(String pStr, Hashtable<String, float[]> pTable)
	{
		try
		{
			if (pTable.get(pStr) != null)
				return true;
		}
		catch (NullPointerException e)
		{
			return false;
		}

		return false;
	}

	/**
	 * Metodo utilizzato per il recupero della base di conoscenza
	 * 
	 * @return restiuisce un oggetto di tipo ElencoCategorie rappresentante la
	 *         base di conoscenza
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	private static ElencoCategorie apriElenco() throws IOException,
			ClassNotFoundException
	{
		File kBase = new File("kb.sbt");// si apre il file kb.sbt
		FileInputStream kBaseStream = new FileInputStream(kBase);// si crea
		// uno stream con il file
		ObjectInputStream kBaseObj = new ObjectInputStream(kBaseStream);// si
		// crea uno stream di oggetti con il file
		ElencoCategorie toReturn;

		toReturn = (ElencoCategorie) kBaseObj.readObject();
		// viene estratto l'ogetto salvato nel file e restituito in output
		return toReturn;
	}

	/**
	 * Metodo utilizzato per la creazione del file. Usato nel training
	 * 
	 * @param path stringa che indica il path in cui creare il file
	 * @return restituisce un ObjectOutputStream al file creato
	 * @throws IOException
	 */
	private static ObjectOutputStream createFile(String path)
			throws IOException
	{
		ObjectOutputStream toReturn;
		File f = new File(path);// si crea il file
		if (f.exists())
			f.delete();

		FileOutputStream fOut = new FileOutputStream(path);
		toReturn = new ObjectOutputStream(fOut);// creazione dello stream

		return toReturn;
	}

	/**
	 * Metodo utilizzato per la creazione della base di conoscenza
	 * 
	 * @throws RemoteException
	 */
	public static void training() throws RemoteException
	{

		String[] elencoCategorie = new String[4];// array contenentei nomi
		// delle categrie da analizzare
		// base di conoscenza
		ElencoCategorie elenco = new ElencoCategorie();

		ObjectOutputStream elencoOut;
		try
		{
			// si tenta la creazione del file
			elencoOut = createFile("kb.sbt");
		}
		catch (IOException e)
		{
			throw new RemoteException("Errore nella creazione del file kb.sbt");
		}

		elencoCategorie[0] = "arte";
		elencoCategorie[1] = "cinema";
		elencoCategorie[2] = "sport";
		// elencoCategorie[3]="";

		for (int i = 0; i < 3; i++)// per ogni categoria
		{
			// si crea un nuovo oggetto di tipo Categoria che conterrà tutti i
			// dati della categoria da analizzare
			Categoria toPutCat = new Categoria(elencoCategorie[i]);
			// se l'inserimento della categoria nella tabella delle categorie
			// non ha successo
			// si lancia un eccezione
			if (!elenco.addCategoria(elencoCategorie[i], toPutCat))
			{
				throw new RemoteException(
						"Errore nella creazione dei dati della categoria "
								+ elencoCategorie[i]);
			}
			// si tenta di leggere dalla cartella contenente i testi di una
			// categoria
			// 100 testi di esempio
			for (Integer j = new Integer(1); j <= 100; j++)
			{
				// path della cartella della categria
				String path = "kb/" + elencoCategorie[i] + "/" + j.toString();
				// si tenta di leggere il file iesimo
				FileReader testoReader;
				try
				{
					testoReader = new FileReader(path);
				}
				catch (FileNotFoundException e)
				{
					// nel caso il file non esista si continua l'esecuzione dal
					// file i+1
					// System.out.println("Errore sul file " + path);
					continue;
				}
				Scanner testoScanner = new Scanner(testoReader);
				while (testoScanner.hasNextLine())
				{
					// si legge il testo del file linea per linea
					String txt = testoScanner.nextLine();
					txt = replaceAndLower(txt);
					String[] toIterate = txt.split(" ");
					for (int k = 0; k < toIterate.length; k++)
					{// per ogni termine della linea
						if (toIterate[k].length() <= 3)// si eliminano i
														// termini poco
														// rilevanti
							continue;
						float[] valTerm, valTotTerm;
						// se il termine è presente nella tabella dei termini
						// della categoria analizzata
						if (elenco.getCategoria(elencoCategorie[i])
								.esisteTermine(toIterate[k]))
						{
							// la sua frequenza è uguale al valore salvato nella
							// tabella più uno fratto il numero totale di
							// termini del testo
							valTerm = elenco.getCategoria(elencoCategorie[i])
									.getVal(toIterate[k]);
							valTerm[0] += (float) 1 / toIterate.length;
							valTotTerm = elenco.getTermine(toIterate[k]);
							valTotTerm[0] += (float) 1 / toIterate.length;
						}
						else
						{
							//altrimenti è uguale ad uno fratto il numero totale dei termini del testo
							valTerm = new float[3];
							valTotTerm = new float[1];
							valTerm[0] = (float) 1 / toIterate.length;
							valTotTerm[0] = (float) 1 / toIterate.length;
						}
						//si salvano i valori calcolati nella tabella dei termini della categoria analizzata
						elenco.setTermine(toIterate[k], valTotTerm);
						elenco.getCategoria(elencoCategorie[i]).addTermine(
								toIterate[k], valTerm);
						if (elenco.getMaxDist() < valTotTerm[0])
							elenco.setMaxDist(valTotTerm[0]);
					}
				}

			}
		}
		for (String nomeCategoria : elenco.Categorie())
		{//per ogni categoria 
			//viene prelavata la tabella dei termini
			Hashtable<String, float[]> terminiCategoria = elenco.getCategoria(
					nomeCategoria).getTermini();

			//vengono analizzati tutti i termini presenti nella tabella dei termini caricata
			for (Enumeration<String> enumTerm = terminiCategoria.keys(); enumTerm
					.hasMoreElements();)
			{
				//viene effattuato il calcolo di rilevanza e distanza
				String termine = enumTerm.nextElement();
				float[] val = terminiCategoria.get(termine);
				val[1] = distanza(termine, elenco.getCategoria(nomeCategoria),
						elenco.getTotTermini());
				val[2] = rilevanza(val[1], elenco.getMaxDist());
			}
			//i dati vengono salvati nella tabella dei termini della categoria
			elenco.getCategoria(nomeCategoria).setTermini(terminiCategoria);
		}
		try
		{//si scrive il risultato delle operazioni su file
			elencoOut.writeObject(elenco);
		}
		catch (IOException e)
		{
			throw new RemoteException("Errore nella scrittura del file!");
		}
	}
}
